/* gpsk31  - PSK31 for Linux with a GTK+ Interface
 * Copyright (C) 2000 Luc Langehegermann, LX2GT
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

/*
 * Misc Functions
 */

#include <time.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "misc.h"
#include "spectrum.h"
#include "main_screen.h"
#include "globals.h"
#include "server.h"
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "text.h"
#include "phasescope.h"
#include "menu.h"

int OPTnew;

extern int display_waterfall;	/* display - 0: spectrum, 1: waterfall */
extern int waterfall_colored;	/* 0 b/w, 1 color mode */

gint				/* Will be called when we click on the spectrum */
  spectrum_motion_event (GtkWidget * widget, GdkEventMotion * event)
{				// Click on the spectrum
  GdkModifierType state;
  GdkEventType    type;
  int x;
  int know_button=0;
  float newfreq;
   
  type=event->type;

  if (event->is_hint) {
    int d1,d2;
    /* figure out the button pressed - if possible */
    gdk_window_get_pointer (event->window, &d1, &d2, &state);
    know_button=1;
  }  

  if ((know_button == 0) || 
      ((type == GDK_BUTTON_PRESS) && (state & GDK_BUTTON1_MASK))) {
    /* left button: set RX freq */
    x = (int) event->x;
    newfreq = spectrum_new_fc (x);
    commControl (COMM_RXCH, COMM_FREQ, (int) (100 * newfreq));
  } else if ((type == GDK_BUTTON_PRESS) && (state & GDK_BUTTON3_MASK)) {
    /* right button: toggle between Waterfall & spectrum */
     if (display_waterfall) {
       display_waterfall=0;
     } else {
       display_waterfall=1;
     }
  } else if ((type == GDK_BUTTON_PRESS) && (state & GDK_BUTTON2_MASK)) {
    /* middle click. toggle b/w - color waterfall */
    if (waterfall_colored) {
      waterfall_colored=0;
    } else {
      waterfall_colored=1;
    }
  }


  return 0;
}

gint display_time (gpointer data)	/* Displays the UTC-Date */
{
  char *c;
  c = get_time_string (NULL);
  gtk_statusbar_pop (GTK_STATUSBAR (statusbar.time), 1);
  gtk_statusbar_push (GTK_STATUSBAR (statusbar.time), 1, c);
  g_free (c);
  return TRUE;
}

/*
 * This function returns a pointer to the time string
 * YOU MUST FREE it
 */


gchar *
get_time_string (gchar * format)
{
  time_t calendar_time;
  struct tm *my_time;
  char time_string[50];

  calendar_time = time (0);
  my_time = gmtime (&calendar_time);
  if (format == NULL)
    strftime (time_string, 50, "%a, %d %b %H:%M:%Sz %Y", my_time);
  else
    strftime (time_string, 50, format, my_time);
  return g_strdup (time_string);
}

/*
 * This function parses the configuration file
 */

void
read_config ()
{
  int i, fd;
  gchar *temp;

  char *home = getenv ("HOME");
  char conf[strlen (home) + 60];

  strcpy (conf, home);
  strcat (conf, "/.gpsk31/gpsk.conf");
  fd = open (conf, O_RDONLY);
  if (fd == -1)
    fd = open (DATADIR "/gpsk.conf", O_RDONLY);
  if (fd == -1)
    warning_dialog ("Unable to open configuration file!", TRUE);
  GScanner *scanner = g_scanner_new (NULL);
  char string0[16][40];
  char string1[16][40];
  scanner->config->numbers_2_int = TRUE;
  scanner->config->symbol_2_token = TRUE;


/* Hint:
 enum [tag] {name [=value], ...};

 The set can optionally be given a type tag name with tag.
 name is the name of a constant that can optionally be assigned
 the (constant) value of value, etc. For example,

 enum Numbers {One = 1, Two = 2, Three = 3, Four = 4, Five = 5};

 If value is missing, then a value is assumed to be the value of
 the previous constant in the list + 1.
 If this is the first constant in the list, the default value is 0.
*/
  enum
  { SYMBOL_TX_WINDOW_COLOR = G_TOKEN_LAST + 1, /* start with G_TOKEN_LAST+1 */
    SYMBOL_RX_WINDOW_COLOR_RX,
    SYMBOL_RX_WINDOW_COLOR_TX,
    SYMBOL_TX_WINDOW_FONT,
    SYMBOL_RX_WINDOW_FONT,
    SYMBOL_PHASE_SCOPE_DCD_ON_COLOR,
    SYMBOL_PHASE_SCOPE_DCD_OFF_COLOR,
    SYMBOL_PHASE_SCOPE_ARC_COLOR,
    SYMBOL_SPECTRUM_COLOR,
    SYMBOL_SPECTRUM_TUNE_LINE_COLOR,
    SYMBOL_SPECTRUM_CENTER_COLOR,
    SYMBOL_CALLSIGN,
    SYMBOL_NAME,
    SYMBOL_QTH,
    SYMBOL_CW_STRING,
    SYMBOL_FREQ,
    SYMBOL_DCD_LEVEL,
    SYMBOL_LSB,
    SYMBOL_PTT_DEVICE,
    SYMBOL_AUDIO_DEVICE,
    SYMBOL_NEW_RX,
    SYMBOL_SPECTRUM_WIDTH,
    SYMBOL_SPECTRUM_SPEED,
    SYMBOL_FIRST_FIX,
    SYMBOL_LOG_FILENAME
  };

  g_scanner_scope_add_symbol (scanner, 0, "tx_window_color",
			GINT_TO_POINTER (SYMBOL_TX_WINDOW_COLOR));
  g_scanner_scope_add_symbol (scanner, 0, "rx_window_color_rx",
			GINT_TO_POINTER (SYMBOL_RX_WINDOW_COLOR_RX));
  g_scanner_scope_add_symbol (scanner, 0, "rx_window_color_tx",
			GINT_TO_POINTER (SYMBOL_RX_WINDOW_COLOR_TX));
  g_scanner_scope_add_symbol (scanner, 0, "tx_window_font",
			GINT_TO_POINTER (SYMBOL_TX_WINDOW_FONT));
  g_scanner_scope_add_symbol (scanner, 0, "rx_window_font",
			GINT_TO_POINTER (SYMBOL_RX_WINDOW_FONT));
  g_scanner_scope_add_symbol (scanner, 0, "phase_scope_dcd_on_color",
			GINT_TO_POINTER (SYMBOL_PHASE_SCOPE_DCD_ON_COLOR));
  g_scanner_scope_add_symbol (scanner, 0, "phase_scope_dcd_off_color",
			GINT_TO_POINTER (SYMBOL_PHASE_SCOPE_DCD_OFF_COLOR));
  g_scanner_scope_add_symbol (scanner, 0, "phase_scope_arc_color",
			GINT_TO_POINTER (SYMBOL_PHASE_SCOPE_ARC_COLOR));
  g_scanner_scope_add_symbol (scanner, 0, "spectrum_color",
			GINT_TO_POINTER (SYMBOL_SPECTRUM_COLOR));
  g_scanner_scope_add_symbol (scanner, 0, "spectrum_tune_line_color",
			GINT_TO_POINTER (SYMBOL_SPECTRUM_TUNE_LINE_COLOR));
  g_scanner_scope_add_symbol (scanner, 0, "callsign",
			GINT_TO_POINTER (SYMBOL_CALLSIGN));
  g_scanner_scope_add_symbol (scanner, 0, "name", GINT_TO_POINTER (SYMBOL_NAME));
  g_scanner_scope_add_symbol (scanner, 0, "qth", GINT_TO_POINTER (SYMBOL_QTH));
  g_scanner_scope_add_symbol (scanner, 0, "cw_string",
			GINT_TO_POINTER (SYMBOL_CW_STRING));
  g_scanner_scope_add_symbol (scanner, 0, "freq", GINT_TO_POINTER (SYMBOL_FREQ));
  g_scanner_scope_add_symbol (scanner, 0, "dcd_level",
			GINT_TO_POINTER (SYMBOL_DCD_LEVEL));
  g_scanner_scope_add_symbol (scanner, 0, "lsb", GINT_TO_POINTER (SYMBOL_LSB));
  g_scanner_scope_add_symbol (scanner, 0, "ptt_device",
			GINT_TO_POINTER (SYMBOL_PTT_DEVICE));
  g_scanner_scope_add_symbol (scanner, 0, "audio_device",
			GINT_TO_POINTER (SYMBOL_AUDIO_DEVICE));
  g_scanner_scope_add_symbol (scanner, 0, "new_rx", GINT_TO_POINTER (SYMBOL_NEW_RX));
  g_scanner_scope_add_symbol (scanner, 0, "spectrum_width",
			GINT_TO_POINTER (SYMBOL_SPECTRUM_WIDTH));
  g_scanner_scope_add_symbol (scanner, 0, "spectrum_speed",
			GINT_TO_POINTER (SYMBOL_SPECTRUM_SPEED));
  g_scanner_scope_add_symbol (scanner, 0, "log_filename",
			GINT_TO_POINTER (SYMBOL_LOG_FILENAME));
  g_scanner_scope_add_symbol (scanner, 0, "spectrum_center_line_color",
			GINT_TO_POINTER (SYMBOL_SPECTRUM_CENTER_COLOR));

  for (i = SYMBOL_FIRST_FIX; i <= SYMBOL_FIRST_FIX + 30; i = i + 2)
    {
      sprintf (string0[(i - SYMBOL_FIRST_FIX) / 2], "label%d",
	       (i - SYMBOL_FIRST_FIX) / 2);
      g_scanner_scope_add_symbol (scanner, 0, string0[(i - SYMBOL_FIRST_FIX) / 2],
			    GINT_TO_POINTER (i));
      sprintf (string1[(i - SYMBOL_FIRST_FIX) / 2], "text%d",
	       (i - SYMBOL_FIRST_FIX) / 2);
      g_scanner_scope_add_symbol (scanner, 0, string1[(i - SYMBOL_FIRST_FIX) / 2],
			    GINT_TO_POINTER (i + 1));
    }

  g_scanner_input_file (scanner, fd);

  /* tx_window_color */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_TX_WINDOW_COLOR)
    g_error ("gpsk.conf:%d `tx_window_color' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
   ini_settings.tx_window_color = g_strdup(scanner->value.v_string);

  /* rx_window_color_rx */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_RX_WINDOW_COLOR_RX)
    g_error ("gpsk.conf:%d `rx_window_color_rx' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.rx_window_color_rx = g_strdup(scanner->value.v_string);

  /* rx_window_color_tx */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_RX_WINDOW_COLOR_TX)
    g_error ("gpsk.conf:%d `rx_window_color_tx' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.rx_window_color_tx = g_strdup(scanner->value.v_string);


  /* tx_window_font */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_TX_WINDOW_FONT)
    g_error ("gpsk.conf:%d `tx_window_font' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.tx_window_font = g_strdup(scanner->value.v_string);

  /* rx_window_font */
  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_RX_WINDOW_FONT)
    g_error ("gpsk.conf:%d `rx_window_font' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.rx_window_font = g_strdup(scanner->value.v_string);

  /* phase_scope_dcd_on_color */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_PHASE_SCOPE_DCD_ON_COLOR)
    g_error ("gpsk.conf:%d `phase_scope_dcd_on_color' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.phase_scope_dcd_on_color = g_strdup(scanner->value.v_string);

  /* phase_scope_dcd_off_color */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_PHASE_SCOPE_DCD_OFF_COLOR)
    g_error ("gpsk.conf:%d `phase_scope_dcd_off_color' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.phase_scope_dcd_off_color = g_strdup(scanner->value.v_string);

  /* phase_scope_arc_color */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_PHASE_SCOPE_ARC_COLOR)
    g_error ("gpsk.conf:%d `phase_scope_arc_color' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.phase_scope_arc_color = g_strdup(scanner->value.v_string);

  /* Color of the Spectrum Display */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_SPECTRUM_COLOR)
    g_error ("gpsk.conf:%d `spectrum_color' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.spectrum_color = g_strdup(scanner->value.v_string);

  /* Color of the spectrum tune line */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_SPECTRUM_TUNE_LINE_COLOR)
    g_error ("gpsk.conf:%d `spectrum_tune_line_color' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.spectrum_tune_line_color = g_strdup(scanner->value.v_string);


  /* Center frequency color line */
  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_SPECTRUM_CENTER_COLOR) {
    temp = g_strdup_printf (
	  "<b>gpsk.conf: %d `spectrum_center_line_color' expected! "
	  "See the ChangeLog. Exiting program...</b>",
	  g_scanner_cur_line (scanner));
    warning_dialog (temp, TRUE);
    g_free (temp);
  } else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.spectrum_center_line_color = g_strdup(scanner->value.v_string);


  /* callsign of the OP */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_CALLSIGN)
    g_error ("gpsk.conf:%d `callsign' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.call = g_strdup(scanner->value.v_string);

  /* Name */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_NAME)
    g_error ("gpsk.conf:%d `name' expected!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.name = g_strdup(scanner->value.v_string);

  /* qth */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_QTH)
    g_error ("gpsk.conf:%d `qth' expected!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.qth = g_strdup(scanner->value.v_string);

  /* cwid string */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_CW_STRING)
    g_error ("gpsk.conf:%d `cw_string' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.cw_string = g_strdup(scanner->value.v_string);

  /* Center freq. */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_FREQ)
    g_error ("gpsk.conf:%d `freq' expected!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_INT)
    g_error ("gpsk.conf:%d: <number> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.freq = scanner->value.v_int;

  /* dcd_level */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_DCD_LEVEL)
    g_error ("gpsk.conf:%d `dcd_level' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_INT)
    g_error ("gpsk.conf:%d: <number> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.dcd_level = scanner->value.v_int;

  /* Use LSB for qpsk */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_LSB)
    g_error ("gpsk.conf:%d `lsb' expected!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_INT)
    g_error ("gpsk.conf:%d: <number> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.lsb = scanner->value.v_int;

  /* ptt device */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_PTT_DEVICE)
    g_error ("gpsk.conf:%d `ptt_device' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.ptt_device = g_strdup(scanner->value.v_string);

  /* audio device */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_AUDIO_DEVICE)
    g_error ("gpsk.conf:%d `audio_device' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.audio_device = g_strdup(scanner->value.v_string);

  /* new_rx */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_NEW_RX)
    g_error ("gpsk.conf:%d `new_rx' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_INT)
    g_error ("gpsk.conf:%d: <number> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.new_rx = scanner->value.v_int;

  /* spectrum_width */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_SPECTRUM_WIDTH)
    g_error ("gpsk.conf:%d `spectrum_width' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_INT)
    g_error ("gpsk.conf:%d: <number> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.spectrum_width = scanner->value.v_int;

  /* spectrum_speed */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_SPECTRUM_SPEED)
    g_error ("gpsk.conf:%d `spectrum_speed' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_INT)
    g_error ("gpsk.conf:%d: <number> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.spectrum_speed = scanner->value.v_int;

  /* Log filename */

  g_scanner_get_next_token (scanner);
  if (scanner->token != (int) SYMBOL_LOG_FILENAME)
    g_error ("gpsk.conf:%d `log_filename' expected!",
	     g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != '=')
    g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
  else
    g_scanner_get_next_token (scanner);
  if (scanner->token != G_TOKEN_STRING)
    g_error ("gpsk.conf:%d: <string> expected!",
	     g_scanner_cur_line (scanner));
  else
    ini_settings.log_filename = g_strdup(scanner->value.v_string);


  /* Fixed Texts */

  for (i = SYMBOL_FIRST_FIX; i <= SYMBOL_FIRST_FIX + 30; i = i + 2)
    {
      g_scanner_get_next_token (scanner);
      if (scanner->token != (int) i)
	g_error ("gpsk.conf:%d `label%d' expected!",
		 g_scanner_cur_line (scanner),
		 (i - SYMBOL_FIRST_FIX) / 2);
      else
	g_scanner_get_next_token (scanner);
      if (scanner->token != '=')
	g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
      else
	g_scanner_get_next_token (scanner);
      if (scanner->token != G_TOKEN_STRING)
	g_error ("gpsk.conf:%d: <string> expected!",
		 g_scanner_cur_line (scanner));
      else
	ini_settings.label[(i - SYMBOL_FIRST_FIX) / 2] =
        g_strdup(scanner->value.v_string);

      g_scanner_get_next_token (scanner);
      if (scanner->token != (int) i + 1)
	g_error ("gpsk.conf:%d `text%d' expected!",
		 g_scanner_cur_line (scanner),
		 (i - SYMBOL_FIRST_FIX) / 2);
      else
	g_scanner_get_next_token (scanner);
      if (scanner->token != '=')
	g_error ("gpsk.conf:%d `=' missing!", g_scanner_cur_line (scanner));
      else
	g_scanner_get_next_token (scanner);
      if (scanner->token != G_TOKEN_STRING)
	g_error ("gpsk.conf:%d: <string> expected!",
		 g_scanner_cur_line (scanner));
      else
	ini_settings.text[(i - SYMBOL_FIRST_FIX) / 2] =
        g_strdup(scanner->value.v_string);

    }
  /* Destroy the scanner and close the file */

  g_scanner_destroy (scanner);
  close (fd);

}

/*
 * This function set all the things
 */

void
set_config ()
{
  int result = 0;
  static int server_started;
  phase_scope_init ();		/* Build the Phase Scope GCs */
  draw_arc ();			/* redraw the phase scope circle */
  if (server_started == 0)
    {
      server_started = 1;
      if (strlen (ini_settings.ptt_device) != 0)
	result = server_main (ini_settings.audio_device, ini_settings.ptt_device, DATADIR);	/* Start Server */
      else
	result = server_main (ini_settings.audio_device, NULL, DATADIR);
    }
  else
    g_print ("PSK31-Server already started!\n");
  if (result == -1) exit (1); /* bail out if server not started */
  spectrum_offset (ini_settings.freq);
  spectrum_setup ();		/* init the spectrum */
  if (ini_settings.spectrum_width == 0)	/* init fft */
    fft_setup (1024, ini_settings.spectrum_speed);
  else if (ini_settings.spectrum_width == 1)
    fft_setup (2048, ini_settings.spectrum_speed);
  else
    fft_setup (4096, ini_settings.spectrum_speed);
  commControl (COMM_TXCH, COMM_FREQ, 100 * ini_settings.freq);	/* TX Freq */
  commControl (COMM_RXCH, COMM_FREQ, 100 * ini_settings.freq);	/* RX Freq */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (main_screen.tx_freq), ini_settings.freq);	/* TX Freq spin button */


  gtk_adjustment_set_value (GTK_ADJUSTMENT (main_screen.dcd_adj),
			    (float) (ini_settings.dcd_level * -1));
  commControl (COMM_TXCH, COMM_LSB, ini_settings.lsb);	/* LSB */
  commControl (COMM_RXCH, COMM_LSB, ini_settings.lsb);
  OPTnew = ini_settings.new_rx;	/* Use new rx? */

	set_button (0);		/* set the buttons labels */
}

/*
 * This functions sets the label in the text-buttons
 */

void set_button (int x)
{
	int i;
	for (i = 0; i<= 7; ++i) {
		if (x == 0)
			gtk_label_set_text (GTK_LABEL
                        (GTK_BIN (main_screen.f_button[i])->child),
                        ini_settings.label[i]);
		else
			gtk_label_set_text (GTK_LABEL
                        (GTK_BIN (main_screen.f_button[i])->child),
                        ini_settings.label[i+8]);
	}
}

